#include <stdio.h>
#include <gdbm.h>
#include <time.h>
#include <string.h>
#include <unistd.h>

#define DBFILE "/var/lib/news/emaildb"
#define OFFICIAL_GATE "f400.n5020.z2.fidonet.org"
#define COMMAND "/usr/bin/metasend -F nobody@p21.f155.n5030.z2.fidonet.org \
-t %s -s \"RU.LINUX FAQ\" -b -e 7bit -f /var/lib/news/faqnotice.txt \
-m \"text/plain; charset=koi8-r\""

#define SENT 1
#define NOTSENT 0

struct s_addr {
	char *from;
	int fromfound;
	char *ftnsender;
	int ftnsenderfound;
};

int sendnotice(char *addr);
int officialgate(char *addr);
int checkaddr(struct s_addr);
void findaddr(void);
void initdb(void);

int main(int argc, char *argv[])
{
	int initflag = 0;
	int c;

	while ((c = getopt (argc, argv, "i")) != -1)
		switch (c)
		{
		case 'i':
			initflag = 1;
			break;
		default:
			abort();
		}
	if (initflag)
		initdb();
	else
		findaddr();
	exit(0);
}

void initdb(void)
{
        GDBM_FILE dbf;
	
	if ((dbf = gdbm_open(DBFILE, 0, GDBM_NEWDB, 0660, NULL)) != NULL)
		gdbm_close(dbf);
	else
		printf("%s\n", gdbm_strerror(gdbm_errno));
}

void findaddr(void)
{
	char *line = NULL;
	int nchars = 0;
	struct s_addr addr = {NULL, 0, NULL, 0};

	while ((getline(&line, &nchars, stdin) > 1)
			&& ((addr.fromfound == 0) || (addr.ftnsenderfound == 0)) ) 
		if ((sscanf(line, "From:%*[^<]<%a[^>]", &addr.from) == 1) ||
				(sscanf(line,"From: %as", &addr.from) == 1))
			addr.fromfound = 1;
		else if (sscanf(line, "X-FTN-Sender:%*[^<]<%a[^>]",&addr.ftnsender) == 1)
			addr.ftnsenderfound = 1;
	checkaddr(addr);
	if (addr.fromfound == 1) {
		addr.fromfound = 0;
		free(addr.from);
	}
	if (addr.ftnsenderfound == 1 ) {
		addr.ftnsenderfound = 0;
		free(addr.ftnsender);
	}
	free(line);
}

int sendnotice(char *addr)
{
	char *sendcommand = NULL;
	GDBM_FILE dbf;
	int retcode;
	int exists;
	datum addrkey;
	datum addrdata;
	char data[] = "";
	
	addrkey.dptr = addr;
	addrkey.dsize = strlen (addr);
	addrdata.dptr = data;
	addrdata.dsize = strlen(data);
	retcode = NOTSENT;
	while ((dbf = gdbm_open(DBFILE, 0, GDBM_READER, 0660, NULL)) == NULL)
		sleep(1);
		
	exists = gdbm_exists(dbf, addrkey);
	gdbm_close(dbf);
			
	if (!exists) {
		while ((dbf = gdbm_open(DBFILE, 0, GDBM_WRITER, 0660, NULL)) == NULL)
			sleep(1);
		if (gdbm_store(dbf, addrkey, addrdata, GDBM_REPLACE) == 0) {
			asprintf(&sendcommand, COMMAND, addrkey.dptr);
			system(sendcommand);
			free(sendcommand);
			retcode = SENT;
		}
		gdbm_close(dbf);
	}
	return retcode;
}

int checkaddr(struct s_addr addr)
{	
	time_t curtime;
	
	if (addr.fromfound)
		if (!addr.ftnsenderfound) {
			if (sendnotice(addr.from) == SENT) {
				curtime = time(NULL);
				printf("RU.LINUX faq notification sent to %s at %s", addr.from, ctime(&curtime));
			}
		}
		else { 
			if (officialgate(addr.ftnsender)) {
	                        if (sendnotice(addr.from) == SENT) {
	                                curtime = time(NULL);
	                                printf("RU.LINUX faq notification sent to %s (FTN-Sender %s) at %s", addr.from, addr.ftnsender,ctime(&curtime));
	                        }
			}
			else {
	                        if (sendnotice(addr.ftnsender) == SENT) {
	                                curtime = time(NULL);
	                                printf("RU.LINUX faq notification sent to FTN-Sender %s (From %s) at %s", addr.ftnsender, addr.from,ctime(&curtime));
	                        }
			}
		}
}

int officialgate(char *addr) {
	if (strstr(strrchr(addr, '@'), OFFICIAL_GATE) != NULL)
		return 1;
	else
		return 0;
}
